//=========================================================================
//Copyright LizardOfOz.
//This is still beta and things might and will change over time.
//
//Check the following links for updates:
//   https://tf2maps.net/members/lizardofoz.39426/#resources
//   https://twitter.com/LizardOfOz_
//   https://youtube.com/LizardOfOz
//
//Credits:
//  LizardOfOz - the original VSH Plugin. VSH Vscript recreation.
//  Maxxy - Saxton Hale's model imitating Jungle Inferno SFM; Custom animations and promotional material.
//  Velly - VFX, animations scripting, technical assistance
//  JPRAS - Mayann Hale body reference, model feedback.
//  MegapiemanPHD - Saxton Hale and Gray Mann voice acting.
//  Yakibomb - give_tf_weapon script bundle (used for Hale's first-person hands model).
//=========================================================================

function Tick()
{
    try { local antiError = TF_TEAM.MERCS; } catch(e) { return; }

    if (IsValidRound())
        FireListeners("tick_only_valid", null);
    FireListeners("tick_always", null);

    return 0.1;
}

function OnScriptHook_OnTakeDamage(params)
{
    if (IsNotValidPlayerOrBuilding(params.const_entity))
        return;
    FireListeners("damage_hook", params);
}

function OnGameEvent_player_hurt(params)
{
    local victim;
    if ((victim = GetPlayerFromParams(params)) == null)
        return;
    local attacker;
    if ((attacker = GetPlayerFromParams(params, "attacker")) == null)
        return;
    FireListeners("player_hurt", [attacker, victim, params]);
}

function OnGameEvent_player_connect(params)
{
    local player;
    if ((player = GetPlayerFromParams(params)) == null)
        return;
    RunWithDelay("EmitSoundOnClient(\"VSH.Intro\", activator)", player, 5);
}

function OnGameEvent_player_team(params)
{
    if (!("userid" in params))
        return null;
    local player = GetPlayerFromUserID(params.userid);
    if (player == null || !player.IsValid() || !player.IsPlayer())
        return;
    if (player == GetHale() && params.team != TF_TEAM.HALE)
        RunWithDelay("activator.SwitchTeam(TF_TEAM.HALE)", player, 0);
    if (player != GetHale() && params.team == TF_TEAM.HALE)
        RunWithDelay("activator.SwitchTeam(TF_TEAM.MERCS)", player, 0);
    if (player != GetHale() && params.team == TF_TEAM.SPECTATORS && params.silent)
    {
        local name = NetProps.GetPropString(player, "m_szNetname");
        Say(null, "Player '"+name+"' was moved to spectators for some reason. Moving them back.", false);
        RunWithDelay("activator.SwitchTeam(TF_TEAM.MERCS)", player, 0);
    }
}

function OnGameEvent_player_changeclass(params)
{
    local player;
    if ((player = GetPlayerFromParams(params)) == null)
        return;
    local currentClass = NetProps.GetPropInt(player, "m_PlayerClass.m_iClass");
    local desiredClass = params["class"];

    if (IsRoundSetup() && IsNotHale(player))
    {
        player.SetPlayerClass(desiredClass)
        player.ForceRegenerateAndRespawn();
    }
    else
    {
        NetProps.SetPropInt(player, "m_PlayerClass.m_iClass", desiredClass);
        RunWithDelay("NetProps.SetPropInt(activator, \"m_PlayerClass.m_iClass\", "+currentClass+")",player,0);
    }
}

function OnGameEvent_player_spawn(params)
{
    local player;
    if ((player = GetPlayerFromParams(params)) == null)
        return;

    FireListeners("spawn", player);
}

function OnGameEvent_player_death(params)
{
    if (params.death_flags & TF_DEATHFLAG.DEAD_RINGER)
        return;
    local player;
    if ((player = GetPlayerFromParams(params)) == null)
        return;

    FireListeners("death", params);
}

function OnGameEvent_rps_taunt_event(params)
{
    if (params.loser == GetHale().entindex())
        RunWithDelay("KillHaleRPS(activator)", PlayerInstanceFromIndex(params.winner), 3);
}

function KillHaleRPS(winner)
{
    local hale = GetHale();
    local attacker = IsValidPlayer(winner) ? winner : hale;
    hale.TakeDamageEx(
        attacker,
        attacker,
        attacker.GetActiveWeapon(),
        Vector(0,0,0),
        hale.GetOrigin(),
        99999,
        0);
}

function OnGameEvent_post_inventory_application(params)
{
    local player;
    if ((player = GetPlayerFromParams(params)) == null || player.GetBotType() == Constants.EBotType.TF_BOT_TYPE)
        return;

    player.GTFW_Cleanup();

    FireListeners("post_inventory", player);
}

__CollectGameEventCallbacks(this);
__CollectEventCallbacks(this, "OnGameEvent_", "GameEventCallbacks", RegisterScriptGameEventListener);