//=========================================================================
//Copyright LizardOfOz.
//This is still beta and things might and will change over time.
//
//Check the following links for updates:
//   https://tf2maps.net/members/lizardofoz.39426/#resources
//   https://twitter.com/LizardOfOz_
//   https://youtube.com/LizardOfOz
//
//Credits:
//  LizardOfOz - the original VSH Plugin. VSH Vscript recreation.
//  Maxxy - Saxton Hale's model imitating Jungle Inferno SFM; Custom animations and promotional material.
//  Velly - VFX, animations scripting, technical assistance
//  JPRAS - Mayann Hale body reference, model feedback.
//  MegapiemanPHD - Saxton Hale and Gray Mann voice acting.
//  Yakibomb - give_tf_weapon script bundle (used for Hale's first-person hands model).
//=========================================================================

haleWillStomp <- false
haleLastTimeOnGround <- 0
startMercCount <- 1
::voiceRNG <- 0
::voiceTime <- 0
lastHolidayPunchTime <- 0
voiceRNG <- 0
lastHaleRageHP <- 9000
rageWasAnnounced <- false

function TickHale(void)
{
    local hale = GetHale();

    TickSetupHealth(hale);
    TickBossBar(hale);
    TickStomp(hale);
    TickKludges(hale);
    TickScreenShake(hale);
}
AddListener("tick_only_valid", TickHale, 1);

function TickSetupHealth(hale)
{
    if (!IsRoundSetup())
        return;
    local maxHealth = CalcHaleMaxHealth(GetTotalPlayerCount() - 1);
    hale.SetHealth(maxHealth);
    hale.SetMaxHealth(maxHealth);
    hale.RemoveCustomAttribute("max health additive bonus");
    hale.AddCustomAttribute("max health additive bonus", maxHealth - 300, -1);
    haleStartHealth = maxHealth;
    startMercCount = GetAliveMercCount();

    local desiredRageCount = GetAliveMercCount() / 5 + 1;
    haleMaxRageDamage = maxHealth / desiredRageCount * 1.2;
    lastHaleRageHP = maxHealth;
}

function TickBossBar(hale)
{
    local bossBar = Entities.FindByClassname(null, "monster_resource");

    if (IsPlayerDead(hale))
    {
        NetProps.SetPropInt(bossBar,"m_iBossHealthPercentageByte", 0);
        return;
    }
    local hpByte = (hale.GetHealth() * 255) / hale.GetMaxHealth();
    if (hpByte == 0 && hale.GetHealth() > 0)
        hpByte = 1;
    if (hpByte > 255)
        hpByte = 255;
    NetProps.SetPropInt(bossBar,"m_iBossHealthPercentageByte", hpByte);
}

function TickStomp(hale)
{
    if (hale.IsJumping())
        haleWillStomp = hale.GetAbsVelocity().LengthSqr() > 90000 //300^2;
    else
    {
        if (haleWillStomp)
        {
            local victim = NetProps.GetPropEntity(hale,"m_hGroundEntity");
            if (IsValidPlayer(victim))
            {
                local currentWeapon = hale.GetActiveWeapon();
                local mannthreadsWeapon = NetProps.GetPropEntityArray(hale, "m_hMyWeapons", 1);
                NetProps.SetPropEntity(hale, "m_hActiveWeapon", mannthreadsWeapon);
                victim.TakeDamageEx(hale, hale, mannthreadsWeapon, Vector(0,0,0), hale.GetOrigin(), 666, 1);
                NetProps.SetPropEntity(hale, "m_hActiveWeapon", currentWeapon);
                EmitAmbientSoundOn("Weapon_Mantreads.Impact", 8, 1, 100, victim);
                EmitAmbientSoundOn("Player.FallDamageDealt", 4, 1, 100, victim);
                DispatchParticleEffect("stomp_text", hale.GetOrigin(), Vector(0,0,0));
            }
        }
        haleWillStomp = false;
    }
}

function TickKludges(hale)
{
    //if (NetProps.GetPropInt(hale, "m_iTauntItemDefIndex") == 167) //todo taunts
    //    hale.RemoveCond(Constants.ETFCond.TF_COND_TAUNTING);

    if (hale.IsInvulnerable())
    {
        hale.RemoveCond(Constants.ETFCond.TF_COND_STUNNED);
        hale.RemoveCond(Constants.ETFCond.TF_COND_TAUNTING);
    }
    hale.RemoveCond(Constants.ETFCond.TF_COND_CRITBOOSTED_PUMPKIN);
    hale.AddCustomAttribute("move speed bonus", GetHaleSpeed(hale), -1);
}

function TickScreenShake(hale)
{
    if (hale.GetAbsVelocity().Length() < 200)
        return;
    local haleOrigin = hale.GetOrigin();
    foreach(target in GetAliveMercs())
    {
        if (target == hale)
            continue;
        local targetOrigin = target.GetOrigin();
        local distance = (haleOrigin - targetOrigin).Length();
        if (distance > 1600)
            continue;
        local shakeFactorAmp = clamp((1600 - distance) / 100, 3, 10) / 2;
        local shakeFactorDur = clamp((1600 - distance) / 1000, 0.1, 1);
        ScreenShake(targetOrigin, shakeFactorAmp, 16, shakeFactorDur, 50, 0, true);
    }
}