//=========================================================================
//Copyright LizardOfOz.
//This is still beta and things might and will change over time.
//
//Check the following links for updates:
//   https://tf2maps.net/members/lizardofoz.39426/#resources
//   https://twitter.com/LizardOfOz_
//   https://youtube.com/LizardOfOz
//
//Credits:
//  LizardOfOz - the original VSH Plugin. VSH Vscript recreation.
//  Maxxy - Saxton Hale's model imitating Jungle Inferno SFM; Custom animations and promotional material.
//  Velly - VFX, animations scripting, technical assistance
//  JPRAS - Mayann Hale body reference, model feedback.
//  MegapiemanPHD - Saxton Hale and Gray Mann voice acting.
//  Yakibomb - give_tf_weapon script bundle (used for Hale's first-person hands model).
//=========================================================================

::mercChangesClasses <- [];
::mercChanges <- {};
class MercChange
{
    lastFireTime = 0;
    lastWeapon = null;
    player = null;

    function constructor(player)
    {
        if (!IsValidPlayer(player))
            return;
        this.player = player;
        if (!CanApply())
            return;
        if (!(player in mercChanges))
            mercChanges[player] <- [];
        mercChanges[player].push(this);
        OnApply();
    }
    function DoTick()
    {
        OnTick();
    }
    function DoFrameTick()
    {
        DoFrameTickWeapon();
        OnFrameTick();
    }
    function DoFrameTickWeapon()
    {
        local weapon = player.GetActiveWeapon();
        if (weapon != lastWeapon)
        {
            lastFireTime = 0;
            lastWeapon = weapon;
        }
        if (weapon != null)
        {
            local lastFireTimeNew = NetProps.GetPropFloat(weapon, "LocalActiveWeaponData.m_flNextPrimaryAttack");
            if (lastFireTimeNew > lastFireTime && lastFireTime > 0 && lastFireTime < Time())
                OnWeaponFired(weapon);
            lastFireTime = lastFireTimeNew;
        }
    }

    function CanApply() { return true; }
    function OnApply() { }
    function OnFrameTick() { }
    function OnTick() { }
    function OnWeaponFired(weapon) { }
    function OnDamageDealt(params) { }
    function OnDamageTaken(params) { }
    function OnDeath(params) { }
}

IncludeScript("vssaxtonhale/mercs/merc_changes/single_class/spy_backstab.nut");
IncludeScript("vssaxtonhale/mercs/merc_changes/single_class/demo_katana.nut");
IncludeScript("vssaxtonhale/mercs/merc_changes/single_class/demo_shield.nut");
IncludeScript("vssaxtonhale/mercs/merc_changes/single_class/demo_sword.nut");
IncludeScript("vssaxtonhale/mercs/merc_changes/single_class/demo_sticky_hooray.nut");
IncludeScript("vssaxtonhale/mercs/merc_changes/single_class/engineer_sentry.nut");
IncludeScript("vssaxtonhale/mercs/merc_changes/single_class/heavy_minigun.nut");
IncludeScript("vssaxtonhale/mercs/merc_changes/single_class/medic_uber_rate.nut");
IncludeScript("vssaxtonhale/mercs/merc_changes/single_class/scout_candy_cane.nut");
IncludeScript("vssaxtonhale/mercs/merc_changes/single_class/sniper_headshot.nut");
IncludeScript("vssaxtonhale/mercs/merc_changes/single_class/soldier_airstrike.nut");
IncludeScript("vssaxtonhale/mercs/merc_changes/single_class/soldier_market_gardener.nut");

IncludeScript("vssaxtonhale/mercs/merc_changes/all_class/last_mann_standing.nut");
IncludeScript("vssaxtonhale/mercs/merc_changes/all_class/melee_crits.nut");
IncludeScript("vssaxtonhale/mercs/merc_changes/all_class/melee_wall_climb.nut");

function MercChanges_Spawn(player)
{
    mercChanges[player] <- [];
    foreach (mercChange in mercChangesClasses)
        mercChange(player);
}
AddListener("spawn", MercChanges_Spawn, -1);

function MercChanges_Tick(void)
{
    foreach (player in GetAliveMercs())
        if (player in mercChanges)
            foreach (mercChange in mercChanges[player])
                mercChange.DoTick();
}
AddListener("tick_always", MercChanges_Tick, 2);

function MercChanges_FrameTick(void)
{
    foreach (player in GetAliveMercs())
        if (player in mercChanges)
            foreach (mercChange in mercChanges[player])
                mercChange.DoFrameTick();
}
AddListener("tick_frame", MercChanges_FrameTick, 2);

function MercChanges_Death(params)
{
    local player;
    if ((player = GetPlayerFromParams(params)) == null || IsHale(player))
        return;
    foreach (mercChange in mercChanges[player])
        mercChange.OnDeath(params);
}
AddListener("death", MercChanges_Death, 0);

function MercChanges_Damage(params)
{
    local victim = params.const_entity;
    local attacker = params.attacker;

    if (IsHale(victim) && IsValidPlayer(attacker))
        foreach (mercChange in mercChanges[attacker])
            mercChange.OnDamageDealt(params);

    else if (IsHale(attacker) && IsValidPlayer(victim))
        foreach (mercChange in mercChanges[victim])
            mercChange.OnDamageTaken(params);
}
AddListener("damage_hook", MercChanges_Damage, 0);