//=========================================================================
//Copyright LizardOfOz.
//This is still beta and things might and will change over time.
//
//Check the following links for updates:
//   https://tf2maps.net/members/lizardofoz.39426/#resources
//   https://twitter.com/LizardOfOz_
//   https://youtube.com/LizardOfOz
//
//Credits:
//  LizardOfOz - the original VSH Plugin. VSH Vscript recreation.
//  Maxxy - Saxton Hale's model imitating Jungle Inferno SFM; Custom animations and promotional material.
//  Velly - VFX, animations scripting, technical assistance
//  JPRAS - Mayann Hale body reference, model feedback.
//  MegapiemanPHD - Saxton Hale and Gray Mann voice acting.
//  Yakibomb - give_tf_weapon script bundle (used for Hale's first-person hands model).
//=========================================================================

function IsPlayerAlive(player)
{
    return NetProps.GetPropInt(player, "m_lifeState") == LIFE_STATE.ALIVE;
}

function IsPlayerDead(player)
{
    return NetProps.GetPropInt(player, "m_lifeState") != LIFE_STATE.ALIVE;
}

::IsValidPlayer <- function(player)
{
    try
    {
        return player != null && player.IsValid() && player.IsPlayer() && player.GetTeam() > 1;
    }
    catch(e)
    {
        return false;
    }
}

::IsNotValidPlayer <- function(player)
{
    try
    {
        return player == null || !player.IsValid() || !player.IsPlayer() || player.GetTeam() <= 1;
    }
    catch (e)
    {
        return true;
    }
}

function IsValidPlayerOrBuilding(entity)
{
    try
    {
        return entity != null
            && entity.IsValid()
            && entity.GetTeam() > 1
            && (entity.IsPlayer() || entity.GetClassname().find("obj_") == 0);
    }
    catch(e)
    {
        return false;
    }
}

function IsNotValidPlayerOrBuilding(entity)
{
    try
    {
        return entity == null
            || !entity.IsValid()
            || entity.GetTeam() <= 1
            || (!entity.IsPlayer() && entity.GetClassname().find("obj_") == null);
    }
    catch(e)
    {
        return true;
    }
}

function IsValidBuilding(building)
{
    try
    {
        return building != null
            && building.IsValid()
            && building.GetClassname().find("obj_") == 0
            && building.GetTeam() > 1;
    }
    catch(e)
    {
        return false;
    }
}

function GetAllValidPlayers()
{
    for (local i = 1; i <= Constants.Server.MAX_PLAYERS; i++)
    {
        local player = PlayerInstanceFromIndex(i);
        if (IsValidPlayer(player))
            yield player;
    }
    return null;
}

function GetAllValidMercs()
{
    for (local i = 1; i <= Constants.Server.MAX_PLAYERS; i++)
    {
        local player = PlayerInstanceFromIndex(i);
        if (IsValidPlayer(player) && IsNotHale(player))
            yield player;
    }
    return null;
}

function GetBuildings()
{
    local building = null;
    while(building = Entities.FindByClassname(building, "obj_*"))
    {
        if (building.IsValid());
            yield building;
    }
    return null;
}

function IsMercAlive(player)
{
    return IsValidPlayer(player) && IsNotHale(player) && IsPlayerAlive(player);
}

function GetAliveMercs()
{
    for (local i = 1; i <= Constants.Server.MAX_PLAYERS; i++)
    {
        local player = PlayerInstanceFromIndex(i);
        if (IsMercAlive(player))
            yield player;
    }
    return null;
}

function GetAliveMercCount()
{
    local aliveCount = 0;
    for (local i = 1; i <= Constants.Server.MAX_PLAYERS; i++)
        if (IsMercAlive(PlayerInstanceFromIndex(i)))
            aliveCount++;
    return aliveCount;
}


function GetTotalPlayerCount()
{
    local aliveCount = 0;
    for (local i = 1; i <= Constants.Server.MAX_PLAYERS; i++)
        if (IsValidPlayer(PlayerInstanceFromIndex(i)))
            aliveCount++;
    return aliveCount;
}

function GetPlayerFromParams(params, key = "userid")
{
    if (!(key in params))
        return null;
    local player = GetPlayerFromUserID(params[key]);
    if (IsValidPlayer(player))
        return player;
    return null;
}

function PlaySoundForAll(soundScript)
{
    for (local i = 1; i <= Constants.Server.MAX_PLAYERS; i++)
    {
        local player = PlayerInstanceFromIndex(i);
        if (IsValidPlayer(player))
            EmitSoundOnClient(soundScript, player);
    }
}

::CTFPlayer.Yeet <- function(vector)
{
    NetProps.SetPropEntity(this, "m_hGroundEntity", null);
    this.ApplyAbsVelocityImpulse(vector);
    this.RemoveFlag(Constants.FPlayer.FL_ONGROUND);
}

::CTFBot.SwitchTeam <- function(team)
{
    this.ForceChangeTeam(team, true);
    NetProps.SetPropInt(this, "m_iTeamNum", team);
}

::CTFPlayer.SwitchTeam <- function(team)
{
    this.ForceChangeTeam(team, true);
    NetProps.SetPropInt(this, "m_iTeamNum", team);
}